'use strict';

const co = require('co');
const sqlTool = require('./sqlTool');

class Model {
    constructor(db, table) {
        this.db = db;
        this.table = table;
        this.query = this.query.bind(this);
        this.find = this.find.bind(this);
        this.findOne = this.findOne.bind(this);
        this.save = this.save.bind(this);
        this.update = this.update.bind(this);
        this.count = this.count.bind(this);
    }

    query(sql, values = '', conn) {
        return new Promise((resolve, reject) => {
            this.db.query(sql, values, conn).then(data => {
                return resolve(data);
            }).catch(err => {
                return reject(err);
            });
        });
    }

    find(params, conn) {
        const options = sqlTool.parseOptions(params);
        const {columns, where, values, sort, limit, offset} = options;
        const sql = `select ${columns} from ${this.table} ${where} ${sort} ${limit} ${offset}`;
        return new Promise((resolve, reject) => {
            this.db.query(sql, values, conn).then(data => {
                return resolve(data);
            }).catch(err => {
                return reject(err);
            });
        });
    }

    findOne(params, conn) {
        const options = sqlTool.parseOptions(params);
        const {columns, where, values, sort, limit, offset} = options;
        const sql = `select ${columns} from ${this.table} ${where} ${sort} ${limit} ${offset}`;
        return new Promise((resolve, reject) => {
            this.db.findOne(sql, values, conn).then(data => {
                return resolve(data);
            }).catch(err => {
                return reject(err);
            });
        });
    }

    save(params, conn) {
        const columns = Object.keys(params);
        const values = Object.keys(params).map(function(key) {
            return params[key];
        });
        const sql = `insert into ${this.table} (${columns.join(',')}) values ('${values.join('\',\'')}')`;
        return new Promise((resolve, reject) => {
            this.db.query(sql, '', conn).then(data => {
                return resolve(data);
            }).catch(err => {
                return reject(err);
            });
        });
    }

    update(params, conn) {
        const options = sqlTool.parseOptions(params);
        const {where, set, values} = options;
        const sql = `update ${this.table} set ${set} ${where}`;
        return new Promise((resolve, reject) => {
            this.db.query(sql, values, conn).then(data => {
                return resolve(data);
            }).catch(err => {
                return reject(err);
            });
        });
    }

    count(params, conn) {
        const options = sqlTool.parseOptions(params);
        const {where, values} = options;
        const sql = `select count(*) as count from ${this.table} ${where}`;
        return new Promise((resolve, reject) => {
            this.db.query(sql, values, conn).then(count => {
                return resolve(count);
            }).catch(err => {
                return reject(err);
            });
        });
    }

    findAndCountAll(params) {
        const self = this;
        const options = sqlTool.parseOptions(params);
        const {columns, where, values, limit, offset} = options;
        return new Promise((resolve, reject) => {
            co(function* () {
                const countAndRows = yield [
                    self.count(params),
                    self.find(params),
                ];
                return resolve({
                    count: countAndRows[0][0].count || 0,
                    rows: countAndRows[1]
                });
            }).catch(err => {
                return reject(err);
            });
        });
    }
}

module.exports = Model;