'use strict';

const sqlTool = {};

sqlTool.parseOptions = (params) => {
    params = params || {};
    let ret = { // 解析后的返回结果对象
        where: '',
        values: [],
        sort: '',
        limit: '',
        offset: '',
        set: '',
        columns: '*'
    };
    if (params.columns && params.columns.length > 0) {
        ret.columns = params.columns.join(',');
    }

    if (_.isObject(params.set) && Object.keys(params.set).length) {
        ret.set = Object.keys(params.set).join(' = ?, ');
        ret.set += ' = ?';
        ret.values = Object.keys(params.set).map((key) => {
            return params.set[key];
        });
    }

    if (_.isObject(params.where) && Object.keys(params.where).length > 0) {
        const where = [];
        for (const key in params.where) {
            if (params.where.hasOwnProperty(key)) {
                const values = params.where[key];
                if (Array.isArray(values)) { // 如： 值id是数组[1,2,3] 解析为 id in (1,2,3)
                    const val = values.join(',');
                    where.push(`${key} in (${val})`);
                } else if (_.isObject(values)) {
                    const map = new Map([
                        ['$gt', '>'],
                        ['$lt', '<'],
                        ['$gte', '>='],
                        ['$lte', '<='],
                        ['$ne', '!='],
                        ['$regex', 'like']
                    ]);
                    for (const _key of Object.keys(values)) {
                        where.push(`${key} ${map.get(_key)} ?`);
                        ret.values.push(values[_key]);
                    }
                } else {
                    where.push(`${key} = ?`);
                    ret.values.push(params.where[key]);
                };
            }
        }
        ret.where = where.length ? 'where ' + where.join(' and ') : '';
    }

    if (_.isObject(params.sort) && Object.keys(params.sort).length > 0) {
        let sort = [];
        for (let key in params.sort) {
            if (params.sort.hasOwnProperty(key)) {
                sort.push(`${key} ${params.sort[key]}`);
            }
        }
        ret.sort = sort.length ? 'order by ' + sort.join(' , ') : '';
    }

    if (params.limit && !isNaN(params.limit)) {
        ret.limit = ' limit ' + params.limit || '';
        // offset 必须在有limit存在的条件下才成立
        if (ret.limit && params.offset && !isNaN(params.offset)) {
            ret.offset = ' offset ' + params.offset || '';
        }
    }
    return ret;
};

sqlTool.wrapSql = (sql, val) => {
    return {
        sql: sql,
        values: val
    };
};

const _ = {
    isObject: (value) => {
        const type = typeof value;
        return value != null && (type == 'object');
    }
};

module.exports = sqlTool;